unit TestForm;

// Remote Access API test program
// by Davide Moretti <dmoretti@iper.net>
//
// Play with it! :-)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Ras,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TfrmTest = class(TForm)
    lstEntries: TListBox;
    btnGetEntries: TButton;
    btnCreateEntry: TButton;
    btnEditEntry: TButton;
    grpSelectedEntry: TGroupBox;
    btnGetParams: TButton;
    btnDialEntry: TButton;
    lblUserName: TLabel;
    lblPassword: TLabel;
    txtUserName: TEdit;
    txtPassword: TEdit;
    btnHangup: TButton;
    optDialMode: TRadioGroup;
    lblAsync: TLabel;
    lblStatus: TLabel;
    btnActiveConn: TButton;
    lvConnections: TListView;
    btnSetParams: TButton;
    txtCopyright: TMemo;
    btnRenameEntry: TButton;
    btnDeleteEntry: TButton;
    Label1: TLabel;
    btnSetEntryProperties: TButton;
    btnGetEntryProperties: TButton;
    procedure btnGetEntriesClick(Sender: TObject);
    procedure btnCreateEntryClick(Sender: TObject);
    procedure btnEditEntryClick(Sender: TObject);
    procedure btnDialEntryClick(Sender: TObject);
    procedure btnHangupClick(Sender: TObject);
    procedure btnGetParamsClick(Sender: TObject);
    procedure lstEntriesClick(Sender: TObject);
    procedure txtUserNameChange(Sender: TObject);
    procedure btnActiveConnClick(Sender: TObject);
    procedure btnSetParamsClick(Sender: TObject);
    procedure btnRenameEntryClick(Sender: TObject);
    procedure btnDeleteEntryClick(Sender: TObject);
    procedure btnSetEntryPropertiesClick(Sender: TObject);
    procedure btnGetEntryPropertiesClick(Sender: TObject);
  private
    { Private declarations }
    dialparams: TRasDialParams;
    procedure set_entry_params(var entry: TRasEntry);
  public
    { Public declarations }
  end;

var
  frmTest: TfrmTest;

implementation

uses ConnProp;

{$R *.DFM}

function StatusString(state: TRasConnState; error: Longint): String;
  var
    c: Array[0..100] of Char;
    s: String;
  begin
  if error <> 0 then
    begin
    RasGetErrorString(error, c, 100);
    Result := c;
    end
  else
    begin
    s := '';
    case State of
      RASCS_OpenPort:
        s := 'Opening port';
      RASCS_PortOpened:
        s := 'Port opened';
      RASCS_ConnectDevice:
        s := 'Connecting device';
      RASCS_DeviceConnected:
        s := 'Device connected';
      RASCS_AllDevicesConnected:
        s := 'All devices connected';
      RASCS_Authenticate:
        s := 'Start authenticating';
      RASCS_AuthNotify:
        s := 'Authentication: notify';
      RASCS_AuthRetry:
        s := 'Authentication: retry';
      RASCS_AuthCallback:
        s := 'Authentication: callback';
      RASCS_AuthChangePassword:
        s := 'Authentication: change password';
      RASCS_AuthProject:
        s := 'Authentication: projecting';
      RASCS_AuthLinkSpeed:
        s := 'Authentication: link speed';
      RASCS_AuthAck:
        s := 'Authentication: acknowledge';
      RASCS_ReAuthenticate:
        s := 'Authentication: reauthenticate';
      RASCS_Authenticated:
        s := 'Authenticated';
      RASCS_PrepareForCallback:
        s := 'Preparing for callback';
      RASCS_WaitForModemReset:
        s := 'Waiting for modem reset';
      RASCS_WaitForCallback:
        s := 'Waiting for callback';
      RASCS_Projected:
        s := 'Projected';
      RASCS_StartAuthentication:
        s := 'Start authentication';
      RASCS_CallbackComplete:
        s := 'Callback complete';
      RASCS_LogonNetwork:
        s := 'Logging on network';

      RASCS_Interactive:
        s := 'Interactive';
      RASCS_RetryAuthentication:
        s := 'Retry Authentication';
      RASCS_CallbackSetByCaller:
        s := 'Callback set by caller';
      RASCS_PasswordExpired:
        s := 'Password expired';

      RASCS_Connected:
        s := 'Connected';
      RASCS_Disconnected:
        s := 'Disconnected';
      end;
    Result := s;
    end;
  end;

// Callback RAS function
procedure RasCallback(msg: Integer; state: TRasConnState;
    error: Longint); stdcall
  begin
  if error = 0 then
    frmTest.lblStatus.Font.Color := clBlue
  else
    frmTest.lblStatus.Font.Color := clRed;
  frmTest.lblStatus.Caption := StatusString(state, error);
  frmTest.btnActiveConn.Click;
  end;

function ip_to_string(const ip: TRasIPAddr): string;
  begin
  with ip do
    Result := Format('%d.%d.%d.%d', [a, b, c, d]);
  end;

function string_to_ip(const s: string): TRasIPAddr;
  var
    t: string;

  function brk: Integer;
    var
      p: Integer;
    begin
    Result := 0;
    p := Pos('.', t);
    if p <> 0 then
      begin
      Result := StrToInt(Copy(t, 1, p - 1));
      t := Copy(t, p + 1, 15);
      end;
    end;

  begin
  t := s;
  Result.a := brk;
  Result.b := brk;
  Result.c := brk;
  Result.d := StrToInt(t);
  end;

procedure TfrmTest.set_entry_params(var entry: TRasEntry);
  var
    devicename: string;
  begin
  with frmConnProp, entry do
    begin
    if chkUseCountryAndAreaCodes.Checked then
      dwfOptions := dwfOptions or RASEO_UseCountryAndAreaCodes
    else
      dwfOptions := dwfOptions and (not RASEO_UseCountryAndAreaCodes);
    if chkSpecificIpAddr.Checked then
      dwfOptions := dwfOptions or RASEO_SpecificIpAddr
    else
      dwfOptions := dwfOptions and (not RASEO_SpecificIpAddr);
    if chkSpecificNameServers.Checked then
      dwfOptions := dwfOptions or RASEO_SpecificNameServers
    else
      dwfOptions := dwfOptions and (not RASEO_SpecificNameServers);
    if chkIpHeaderCompression.Checked then
      dwfOptions := dwfOptions or RASEO_IpHeaderCompression
    else
      dwfOptions := dwfOptions and (not RASEO_IpHeaderCompression);
    if chkRemoteDefaultGateway.Checked then
      dwfOptions := dwfOptions or RASEO_RemoteDefaultGateway
    else
      dwfOptions := dwfOptions and (not RASEO_RemoteDefaultGateway);
    if chkDisableLcpExtensions.Checked then
      dwfOptions := dwfOptions or RASEO_DisableLcpExtensions
    else
      dwfOptions := dwfOptions and (not RASEO_DisableLcpExtensions);
    if chkTerminalBeforeDial.Checked then
      dwfOptions := dwfOptions or RASEO_TerminalBeforeDial
    else
      dwfOptions := dwfOptions and (not RASEO_TerminalBeforeDial);
    if chkTerminalAfterDial.Checked then
      dwfOptions := dwfOptions or RASEO_TerminalAfterDial
    else
      dwfOptions := dwfOptions and (not RASEO_TerminalAfterDial);
    if chkModemLights.Checked then
      dwfOptions := dwfOptions or RASEO_ModemLights
    else
      dwfOptions := dwfOptions and (not RASEO_ModemLights);
    if chkSwCompression.Checked then
      dwfOptions := dwfOptions or RASEO_SwCompression
    else
      dwfOptions := dwfOptions and (not RASEO_SwCompression);
    if chkRequireEncryptedPw.Checked then
      dwfOptions := dwfOptions or RASEO_RequireEncryptedPw
    else
      dwfOptions := dwfOptions and (not RASEO_RequireEncryptedPw);
    if chkRequireMsEncryptedPw.Checked then
      dwfOptions := dwfOptions or RASEO_RequireMsEncryptedPw
    else
      dwfOptions := dwfOptions and (not RASEO_RequireMsEncryptedPw);
    if chkRequireDataEncryption.Checked then
      dwfOptions := dwfOptions or RASEO_RequireDataEncryption
    else
      dwfOptions := dwfOptions and (not RASEO_RequireDataEncryption);
    if chkNetworkLogon.Checked then
      dwfOptions := dwfOptions or RASEO_NetworkLogon
    else
      dwfOptions := dwfOptions and (not RASEO_NetworkLogon);
    if chkUseLogonCredentials.Checked then
      dwfOptions := dwfOptions or RASEO_UseLogonCredentials
    else
      dwfOptions := dwfOptions and (not RASEO_UseLogonCredentials);
    if chkPromoteAlternates.Checked then
      dwfOptions := dwfOptions or RASEO_PromoteAlternates
    else
      dwfOptions := dwfOptions and (not RASEO_PromoteAlternates);
{          if chkSecureLocalFiles.Checked then
      dwfOptions := dwfOptions or RASEO_SecureLocalFiles
    else
      dwfOptions := dwfOptions and (not RASEO_SecureLocalFiles);}

    if chkUseCountryAndAreaCodes.Checked then
      begin
      dwCountryID := StrToInt(txtCountryID.Text);
      dwCountryCode := StrToInt(txtCountryCode.Text);
      StrPCopy(szAreaCode, txtAreaCode.Text);
      end;
    StrPCopy(szLocalPhoneNumber, txtPhoneNumber.Text);

    if chkNetBEUI.Checked then
      dwfNetProtocols := dwfNetProtocols or RASNP_Netbeui
    else
      dwfNetProtocols := dwfNetProtocols and (not RASNP_Netbeui);
    if chkIPX.Checked then
      dwfNetProtocols := dwfNetProtocols or RASNP_Ipx
    else
      dwfNetProtocols := dwfNetProtocols and (not RASNP_Ipx);
    if chkIP.Checked then
      dwfNetProtocols := dwfNetProtocols or RASNP_Ip
    else
      dwfNetProtocols := dwfNetProtocols and (not RASNP_Ip);

    // a little shortcut.. :)
    dwFramingProtocol := 1 shl optFraming.ItemIndex;
    if optFraming.ItemIndex = 1 then  // SLIP
      dwFrameSize := StrToInt(txtFrameSize.Text);
    if chkIP.Checked then
      begin
      ipaddr := string_to_ip(txtIpAddr.Text);
      ipaddrDns := string_to_ip(txtPrimaryDNS.Text);
      ipaddrDnsAlt := string_to_ip(txtSecondaryDNS.Text);
      ipaddrWins := string_to_ip(txtPrimaryWINS.Text);
      ipaddrWinsAlt := string_to_ip(txtSecondaryWINS.Text);
      end;
    StrPCopy(szScript, txtScript.Text);

    devicename := cboDevice.Items.Names[cboDevice.ItemIndex];
    StrPCopy(szDeviceName, devicename);
    StrPCopy(szDeviceType, cboDevice.Items.Values[devicename]);
    end;
  end;

procedure TfrmTest.btnGetEntriesClick(Sender: TObject);
  var
    bufsize: Longint;
    numEntries: Longint;
    entries, p: LPRasEntryName;
    x: Integer;
    res: Integer;
  begin
  btnEditEntry.Enabled := False;
  btnGetParams.Enabled := False;
  txtUserName.Text := '';
  txtPassword.Text := '';
  lstEntries.Items.Clear;
  entries := AllocMem(SizeOf(TRasEntryName));
  entries^.dwSize := SizeOf(TRasEntryName);
  bufsize := SizeOf(TRasEntryName);
  res := RasEnumEntries(nil, nil, entries, bufsize, numEntries);
  if res = ERROR_BUFFER_TOO_SMALL then
    begin
    ReallocMem(entries, bufsize);
    FillChar(entries^, bufsize, 0);
    entries^.dwSize := SizeOf(TRasEntryName);
    res := RasEnumEntries(nil, nil, entries, bufsize, numEntries);
    end;
  if res = 0 then
    begin
    if numEntries > 0 then
      begin
      p := entries;
      for x := 0 to numEntries - 1 do
        begin
        lstEntries.Items.Add(p^.szEntryName);
        Inc(p);
        end;
      lstEntries.ItemIndex := 0;
      lstEntriesClick(lstEntries);
      btnEditEntry.Enabled := True;
      btnGetEntryProperties.Enabled := True;
      btnRenameEntry.Enabled := True;
      btnDeleteEntry.Enabled := True;
      btnGetParams.Enabled := True;
      end;
    end
  else
    ShowMessage('RasEnumEntries failed.');
  FreeMem(entries);
  end;

procedure TfrmTest.btnCreateEntryClick(Sender: TObject);
  begin
  if RasCreatePhonebookEntry(Handle, nil) = 0 then
    btnGetEntries.Click
  else
    ShowMessage('RasCreatePhonebookEntry failed.');
  end;

procedure TfrmTest.btnEditEntryClick(Sender: TObject);
  begin
  if RasEditPhonebookEntry(Handle, nil,
      PChar(lstEntries.Items[lstEntries.ItemIndex])) <> 0 then
    ShowMessage('RasEditPhonebookEntry failed.');
  end;

procedure TfrmTest.btnDialEntryClick(Sender: TObject);
  var
    r: integer;
    c: Array[0..100] of Char;
    hRas: THRasConn;
  begin
  Screen.Cursor := crHourglass;
  FillChar(dialparams, SizeOf(TRasDialParams), 0);
  with dialparams do
    begin
    dwSize := Sizeof(TRasDialParams);
    StrPCopy(szEntryName, lstEntries.Items[lstEntries.ItemIndex]);
    StrPCopy(szUserName, txtUserName.Text);
    StrPCopy(szPassword, txtPassword.Text);
    // You can override phone number here...
    // StrPCopy(szPhoneNumber, 'xxxxxx');
    end;
  hRas := 0;
  if optDialMode.ItemIndex = 0 then
    // Sync dial
    begin
    lblStatus.ParentFont := True;
    lblStatus.Caption := 'Dialing...';
    lblStatus.Refresh;
    r := RasDial(nil, nil, dialparams, 0, nil, hRas);
    if r = 0 then
      begin
      lblStatus.Caption := 'Connected';
      btnActiveConn.Click;
      end
    else
      begin
      lblStatus.Caption := 'Error';
      if hRas <> 0 then
        RasHangUp(hRas);
      end;
    end
   else
    // Async dial
    r := RasDial(nil,   // This field is ignored in Windows95
        nil,  // Phonebook: use default (not used on Win95)
        dialparams,
        0, // use callback function of type RASDIALFUNC
        @RasCallback,   // callback function
        hRas);
  if r <> 0 then
    begin
    RasGetErrorString(r, c, 100);
    ShowMessage('RasDial failed: ' + c);
    end;
  Screen.Cursor := crDefault;
  end;

procedure TfrmTest.btnHangupClick(Sender: TObject);
  var
    hRas: THRasConn;
  begin
  hRas := Longint(lvConnections.ItemFocused.Data);
  if RasHangUp(hRas) = 0 then
    begin
    Sleep(1000);  // wait 1 second
    btnActiveConn.Click;
    end
  else
    ShowMessage('RasHangUp failed.');
  end;

procedure TfrmTest.btnGetParamsClick(Sender: TObject);
  var
    fp: LongBool;
    r: Longint;
    c: Array[0..100] of Char;
  begin
  FillChar(dialparams, SizeOf(TRasDialParams), 0);
  with dialparams do
    begin
    dwSize := Sizeof(TRasDialParams);
    StrPCopy(szEntryName, lstEntries.Items[lstEntries.ItemIndex]);
    end;
  r := RasGetEntryDialParams(nil, dialparams, fp);
  if r = 0 then
    with dialparams do
      begin
      txtUserName.Text := szUserName;
      if fp then
        txtPassword.Text := szPassword;
      end
  else
    begin
    RasGetErrorString(r, c, 100);
    ShowMessage('RasGetEntryDialParams failed: ' + c);
    end;
  end;

procedure TfrmTest.lstEntriesClick(Sender: TObject);
  begin
  grpSelectedEntry.Caption := 'Selected entry: ' +
      lstEntries.Items[lstEntries.ItemIndex];
  end;

procedure TfrmTest.txtUserNameChange(Sender: TObject);
  begin
  if lstEntries.ItemIndex = -1 then
    Exit;
  btnDialEntry.Enabled := (txtUserName.Text <> '');
  btnSetParams.Enabled := btnDialEntry.Enabled;
  end;

procedure TfrmTest.btnActiveConnClick(Sender: TObject);
  var
    bufsize: Longint;
    numEntries: Longint;
    x: Integer;
    entries: Array[1..100] of TRasConn;
    stat: TRasConnStatus;
  begin
  entries[1].dwSize := SizeOf(TRasConn);
  bufsize := SizeOf(TRasConn) * 100;
  btnHangup.Enabled := False;
  lvConnections.Items.Clear;
  FillChar(stat, Sizeof(TRasConnStatus), 0);
  stat.dwSize := Sizeof(TRasConnStatus);
  if RasEnumConnections(@entries[1], bufsize, numEntries) = 0 then
    begin
    if numEntries > 0 then
      begin
      for x := 1 to numEntries do
        with lvConnections.Items.Add, entries[x] do
          begin
          Data := Pointer(hrasconn);
          Caption := szEntryName;
          SubItems.Add(szDeviceType);
          SubItems.Add(szDeviceName);
          RasGetConnectStatus(hrasconn, stat);
          SubItems.Add(StatusString(stat.rasconnstate, stat.dwError));
          end;
      lvConnections.ItemFocused := lvConnections.Items[0];
      btnHangup.Enabled := True;
      end;
    end
  else
    ShowMessage('RasEnumConnections failed.');
  end;

procedure TfrmTest.btnSetParamsClick(Sender: TObject);
  begin
  if Application.MessageBox(PChar('Save parameters for entry ' +
      lstEntries.Items[lstEntries.ItemIndex] + '?'), PChar(Caption),
      MB_ICONWARNING or MB_YESNO) = IDYES then
    begin
    with dialparams do
      begin
      // Note: as stated in WIN32.HLP, the szPhoneNumber cannot be changed by
      // this function. The only way to change it (as far as I know) is to use
      // RasEditPhoneBookEntry and change it manually. Or it's possible to
      // specify szPhoneNumber for the RasDial function.
      dwSize := Sizeof(TRasDialParams);
      StrPCopy(szEntryName, lstEntries.Items[lstEntries.ItemIndex]);
      StrPCopy(szUserName, txtUserName.Text);
      StrPCopy(szPassword, txtPassword.Text);
      end;
    if RasSetEntryDialParams(nil, dialparams, False) <> 0 then
      ShowMessage ('RasSetEntryDialParams failed.');
    end;
  end;

procedure TfrmTest.btnRenameEntryClick(Sender: TObject);
  var
    s: String;
  begin
  s := InputBox('Rename entry', 'Enter new entry name',
      lstEntries.Items[lstEntries.ItemIndex]);
  if s <> '' then
    if RasRenameEntry(nil, PChar(lstEntries.Items[lstEntries.ItemIndex]),
        PChar(s)) <> 0 then
      ShowMessage('RasRenameEntry failed.')
    else
      btnGetEntries.Click;
  end;

procedure TfrmTest.btnDeleteEntryClick(Sender: TObject);
  begin
  if Application.MessageBox(PChar(lstEntries.Items[lstEntries.ItemIndex] +
      ' connection will be removed: Are you sure?'), nil,
      MB_YESNO or MB_ICONWARNING) = IDYES then
  if RasDeleteEntry(nil, PChar(lstEntries.Items[lstEntries.ItemIndex])) <> 0 then
     ShowMessage('RasDeleteEntry failed.')
    else
      btnGetEntries.Click;
  end;

procedure GetCountry;
  var
    bufsize: Longint;
    buf: Array[0..255] of Char;     // This is the minimum buffer size as stated by Microsoft
    myCountry: LPRasCtryInfo;
    retVal: Integer;
    cname: String;
  begin
  bufSize := 256;
  FillChar(buf, 256, #0);
  myCountry := @buf[0];   // point myCountry at beginning of buffer
  myCountry^.dwsize := sizeof(TRasCtryInfo);
  // Now, you should set the dwCountryID to the ID of the country you want to get
  // informations (countrycode and name).
  // If you want to enumerate all countries, you must set the dwCountryID to 1
  // and you'll receive the first country.
  // then in the field dwNextCountryID you'll receive the next available country.
  // It will be 0 when no more countries are available.
  //
  // Get the first country in this example
  myCountry^.dwCountryID := 1;
  retVal := RasGetCountryInfo(myCountry, bufSize);
  if retVal = SUCCESS then
    // now you can examine the dwCountryCode and get the country name:
    cname := buf + myCountry^.dwCountryNameOffset;
  //...
  end;

procedure TfrmTest.btnSetEntryPropertiesClick(Sender: TObject);
  var
    entry: TRasEntry;
  begin
  frmConnProp := TfrmConnProp.Create(Self);
  frmConnProp.txtName.Text := 'Connection';
  if frmConnProp.ShowModal = mrOk then
    begin
    FillChar(entry, SizeOf(TRasEntry), 0);
    entry.dwSize := SizeOf(TRasEntry);
    set_entry_params(entry);
    if RasSetEntryProperties(nil, PChar(frmConnProp.txtName.Text),
        @entry, SizeOf(TRasEntry), nil, 0) = 0 then
      btnGetEntries.Click
    else
      ShowMessage('RasSetEntryProperties failed.')
    end;
  frmConnProp.Free;
  end;

procedure TfrmTest.btnGetEntryPropertiesClick(Sender: TObject);
  var
    entry: LPRasEntry;
    entrySize, devinfoSize: Integer;
    entryName: string;
    devicename: string;
    i: Integer;
  begin
  // Get required size for buffers:
  entryName := lstEntries.Items[lstEntries.ItemIndex];
  entrySize := 0;
  devinfoSize := 0;
  if RasGetEntryProperties(nil, PChar(entryName),
      nil, entrySize, nil, devinfoSize) <> ERROR_BUFFER_TOO_SMALL then
    begin
    ShowMessage('RasGetEntryProperties failed.');
    Exit;
    end;
  entry := AllocMem(entrySize);
  try
    entry^.dwSize := SizeOf(TRasEntry);
    if RasGetEntryProperties(nil, PChar(entryName),
        entry, entrySize, nil, devinfoSize) = 0 then
      begin
      frmConnProp := TfrmConnProp.Create(Self);
      with frmConnProp, entry^ do
        begin
        txtName.Text := entryName;
        chkUseCountryAndAreaCodes.Checked := (dwfOptions and RASEO_UseCountryAndAreaCodes) <> 0;
        chkSpecificIpAddr.Checked := (dwfOptions and RASEO_SpecificIpAddr) <> 0;
        chkSpecificNameServers.Checked := (dwfOptions and RASEO_SpecificNameServers) <> 0;
        chkIpHeaderCompression.Checked := (dwfOptions and RASEO_IpHeaderCompression) <> 0;
        chkRemoteDefaultGateway.Checked := (dwfOptions and RASEO_RemoteDefaultGateway) <> 0;
        chkDisableLcpExtensions.Checked := (dwfOptions and RASEO_DisableLcpExtensions) <> 0;
        chkTerminalBeforeDial.Checked := (dwfOptions and RASEO_TerminalBeforeDial) <> 0;
        chkTerminalAfterDial.Checked := (dwfOptions and RASEO_TerminalAfterDial) <> 0;
        chkModemLights.Checked := (dwfOptions and RASEO_ModemLights) <> 0;
        chkSwCompression.Checked := (dwfOptions and RASEO_SwCompression) <> 0;
        chkRequireEncryptedPw.Checked := (dwfOptions and RASEO_RequireEncryptedPw) <> 0;
        chkRequireMsEncryptedPw.Checked := (dwfOptions and RASEO_RequireMsEncryptedPw) <> 0;
        chkRequireDataEncryption.Checked := (dwfOptions and RASEO_RequireDataEncryption) <> 0;
        chkNetworkLogon.Checked := (dwfOptions and RASEO_NetworkLogon) <> 0;
        chkUseLogonCredentials.Checked := (dwfOptions and RASEO_UseLogonCredentials) <> 0;
        chkPromoteAlternates.Checked := (dwfOptions and RASEO_PromoteAlternates) <> 0;
//        chkSecureLocalFiles.Checked := (dwfOptions and RASEO_SecureLocalFiles) <> 0;

        txtCountryID.Text := IntToStr(dwCountryID);
        txtCountryCode.Text := IntToStr(dwCountryCode);
        txtAreaCode.Text := szAreaCode;
        txtPhoneNumber.Text := szLocalPhoneNumber;

        chkNetBEUI.Checked := (dwfNetProtocols and RASNP_Netbeui) <> 0;
        chkIPX.Checked := (dwfNetProtocols and RASNP_Ipx) <> 0;
        chkIP.Checked := (dwfNetProtocols and RASNP_Ip) <> 0;

        case dwFramingProtocol of
          RASFP_Ppp:
            optFraming.ItemIndex := 0;
          RASFP_Slip:
            optFraming.ItemIndex := 1;
          RASFP_Ras:
            optFraming.ItemIndex := 2;
          end;
        txtFrameSize.Text := IntToStr(dwFrameSize);
        txtIpAddr.Text := ip_to_string(ipaddr);
        txtPrimaryDNS.Text := ip_to_string(ipaddrDns);
        txtSecondaryDNS.Text := ip_to_string(ipaddrDnsAlt);
        txtPrimaryWINS.Text := ip_to_string(ipaddrWins);
        txtSecondaryWINS.Text := ip_to_string(ipaddrWinsAlt);
        txtScript.Text := szScript;
        devicename := szDeviceName;
        for i := 0 to cboDevice.Items.Count do
          if cboDevice.Items.Names[i] = devicename then
            begin
            cboDevice.ItemIndex := i;
            Break;
            end;
        end;
      if frmConnProp.ShowModal = mrOk then
        begin
        set_entry_params(entry^);
        if RasSetEntryProperties(nil, PChar(entryName), entry, entrySize, nil, 0) <> 0 then
          ShowMessage('RasSetEntryProperties failed.')
        end;
      frmConnProp.Free;
      end
    else
      ShowMessage('RasGetEntryProperties failed.')
  finally
    FreeMem(entry);
    end;
  end;

end.
